package com.fs.quartz.client;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import cn.hutool.json.JSONUtil;
import com.fs.common.core.domain.AjaxResult;
import com.fs.quartz.domain.SysJob;
import org.springframework.stereotype.Component;


import java.util.Arrays;
import java.util.stream.Collectors;

/**
 * 使用 Hutool 调用 SysJobController API 的客户端.
 * 假设 SysJob 和 AjaxResult DTO 已在 classpath 中.
 */
@Component
public class JobApiClient {

    private String baseUrl = "http://127.0.0.1:7014/monitor/job";

    /**
     * 准备 HttpRequest 对象，设置通用头部信息.
     */
    private HttpRequest prepareRequest(HttpRequest request) {
        request.header("Content-Type", "application/json;charset=UTF-8");
        return request;
    }

    /**
     * 处理 HTTP 响应，将其转换为 AjaxResult.
     */
    private AjaxResult handleResponse(HttpResponse response) {
        String body = response.body();
        if (response.getStatus() == HttpStatus.HTTP_OK || response.getStatus() == HttpStatus.HTTP_CREATED) { // 200 OK or 201 Created
            if (JSONUtil.isJson(body)) {
                return JSONUtil.toBean(body, AjaxResult.class);
            } else {
                System.err.println("服务器返回成功状态码，但响应体不是JSON格式: " + body);
                return AjaxResult.error(response.getStatus(), "服务器返回了非JSON的成功响应: " + response.getStatus());
            }
        } else {
            System.err.println("请求失败，状态码: " + response.getStatus() + ", 响应体: " + body);
            if (JSONUtil.isJson(body)) {
                try {
                    return JSONUtil.toBean(body, AjaxResult.class);
                } catch (Exception e) {
                    return AjaxResult.error(response.getStatus(), "请求失败: " + response.getStatus() + ", 详情: " + body);
                }
            }
            return AjaxResult.error(response.getStatus(), "请求失败: " + response.getStatus() + ", 响应体: " + body);
        }
    }

    /**
     * 新增定时任务
     * 对应: @PostMapping
     * @param sysJob 要新增的任务信息
     * @return AjaxResult 响应结果
     */
    public AjaxResult addJob(SysJob sysJob) {
        String url = this.baseUrl;
        HttpRequest request = prepareRequest(HttpRequest.post(url));
        request.body(JSONUtil.toJsonStr(sysJob));

        try (HttpResponse response = request.execute()) {
            return handleResponse(response);
        } catch (Exception e) {
            System.err.println("调用 addJob 时发生错误: " + e.getMessage());
            return AjaxResult.error("客户端请求异常: " + e.getMessage());
        }
    }

    /**
     * 修改定时任务
     * 对应: @PutMapping
     * @param sysJob 要修改的任务信息 (必须包含 jobId)
     * @return AjaxResult 响应结果
     */
    public AjaxResult editJob(SysJob sysJob) {
        String url = this.baseUrl;
        HttpRequest request = prepareRequest(HttpRequest.put(url));
        request.body(JSONUtil.toJsonStr(sysJob));

        try (HttpResponse response = request.execute()) {
            return handleResponse(response);
        } catch (Exception e) {
            System.err.println("调用 editJob 时发生错误: " + e.getMessage());
            return AjaxResult.error("客户端请求异常: " + e.getMessage());
        }
    }

    /**
     * 定时任务状态修改
     * 对应: @PutMapping("/changeStatus")
     * @param job 包含 jobId 和要更新的 status 的任务对象
     * @return AjaxResult 响应结果
     */
    public AjaxResult changeJobStatus(SysJob job) {
        String url = this.baseUrl + "/changeStatus";
        HttpRequest request = prepareRequest(HttpRequest.put(url));

        // 后端 Controller 根据 jobId 查询，然后更新 status
        // 因此 payload 只需要 jobId 和 status
        SysJob payload = new SysJob();
        payload.setJobId(job.getJobId());
        payload.setStatus(job.getStatus());
        request.body(JSONUtil.toJsonStr(payload));

        try (HttpResponse response = request.execute()) {
            return handleResponse(response);
        } catch (Exception e) {
            System.err.println("调用 changeJobStatus 时发生错误: " + e.getMessage());
            return AjaxResult.error("客户端请求异常: " + e.getMessage());
        }
    }

    /**
     * 定时任务立即执行一次
     * 对应: @PutMapping("/run")
     * @param job 包含 jobId 的任务对象
     * @return AjaxResult 响应结果
     */
    public AjaxResult runJobNow(SysJob job) {
        String url = this.baseUrl + "/run";
        HttpRequest request = prepareRequest(HttpRequest.put(url));

        // 后端 Controller 的 jobService.run(job) 应该主要需要 jobId
        SysJob payload = new SysJob();
        payload.setJobId(job.getJobId());
        request.body(JSONUtil.toJsonStr(payload));

        try (HttpResponse response = request.execute()) {
            return handleResponse(response);
        } catch (Exception e) {
            System.err.println("调用 runJobNow 时发生错误: " + e.getMessage());
            return AjaxResult.error("客户端请求异常: " + e.getMessage());
        }
    }

    /**
     * 删除定时任务
     * 对应: @DeleteMapping("/{jobIds}")
     * @param jobIds 要删除的任务 ID 数组
     * @return AjaxResult 响应结果
     */
    public AjaxResult removeJobs(Long[] jobIds) {
        if (ArrayUtil.isEmpty(jobIds)) {
            return AjaxResult.error("删除操作的任务ID不能为空");
        }
        String jobIdsString = Arrays.stream(jobIds)
                                    .map(String::valueOf)
                                    .collect(Collectors.joining(","));

        String url = this.baseUrl + "/" + jobIdsString;
        HttpRequest request = prepareRequest(HttpRequest.delete(url));

        try (HttpResponse response = request.execute()) {
            return handleResponse(response);
        } catch (Exception e) {
            System.err.println("调用 removeJobs 时发生错误: " + e.getMessage());
            return AjaxResult.error("客户端请求异常: " + e.getMessage());
        }
    }
}
